import WidgetManagerProvider from './plugin';
import * as output from './output';
export default WidgetManagerProvider;
export { registerWidgetManager } from './plugin';
export { WidgetManager } from './manager';
export { WidgetRenderer } from './renderer';
export { output };
