export const v1 = {
    state: import('./v1/state.schema.json'),
    view: import('./v1/view.schema.json')
};
export const v2 = {
    state: import('./v2/state.schema.json'),
    view: import('./v2/view.schema.json')
};
