export * from "./htmlmanager";
export { requireLoader } from "./libembed-amd";
export declare const version: any;
export declare function generateEmbedScript(widgetState: any, imageDataUrl: string): string;
