# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cyc_next_app',
 'cyc_next_app.lib.@jupyter-widgets.schema',
 'cyc_next_app.server',
 'cyc_next_app.server.python',
 'cyc_next_app.server.python.cassist',
 'cyc_next_app.server.python.code_editor',
 'cyc_next_app.server.python.dataframe_manager',
 'cyc_next_app.server.python.experiment_manager',
 'cyc_next_app.server.python.file_explorer',
 'cyc_next_app.server.python.file_manager',
 'cyc_next_app.server.python.libs',
 'cyc_next_app.server.python.model_manager',
 'cyc_next_app.server.python.project_manager',
 'cyc_next_app.server.python.user_space',
 'cyc_next_app.server.python.user_space.ipython',
 'cyc_next_app.server.tests',
 'cyc_next_app.server.tests.test_servers']

package_data = \
{'': ['*'],
 'cyc_next_app': ['lib/@jupyter-widgets/base/*',
                  'lib/@jupyter-widgets/base/css/*',
                  'lib/@jupyter-widgets/base/lib/*',
                  'lib/@jupyter-widgets/controls/*',
                  'lib/@jupyter-widgets/controls/css/*',
                  'lib/@jupyter-widgets/controls/lib/*',
                  'lib/@jupyter-widgets/controls/lib/phosphor/*',
                  'lib/@jupyter-widgets/html-manager/*',
                  'lib/@jupyter-widgets/html-manager/css/*',
                  'lib/@jupyter-widgets/html-manager/lib/*',
                  'lib/@jupyter-widgets/jupyterlab-manager/*',
                  'lib/@jupyter-widgets/jupyterlab-manager/lib/*',
                  'lib/@jupyter-widgets/jupyterlab-manager/schema/*',
                  'lib/@jupyter-widgets/output/*',
                  'lib/@jupyter-widgets/output/lib/*',
                  'lib/@jupyter-widgets/schema/v1/*',
                  'lib/@jupyter-widgets/schema/v2/*',
                  'lib/cassist/*',
                  'lib/codemirror/autocomplete-lsp/*',
                  'lib/codemirror/autocomplete-lsp/autocomplete/*',
                  'lib/codemirror/basic-setup/*',
                  'lib/codemirror/grammar/*',
                  'lib/codemirror/tooltip/*',
                  'lib/components/*',
                  'lib/components/code-panel/*',
                  'lib/components/dataframe-manager/*',
                  'lib/components/error-boundary/*',
                  'lib/components/file-manager/*',
                  'lib/components/hotkeys/*',
                  'lib/components/kernel-manager/*',
                  'lib/components/libs/*',
                  'lib/components/richoutput-panel/*',
                  'lib/components/richoutput-panel/data-panel/*',
                  'lib/components/richoutput-panel/experiment-panel/*',
                  'lib/components/richoutput-panel/model-panel/*',
                  'lib/components/richoutput-panel/result-panel/*',
                  'lib/components/richoutput-panel/summary-panel/*',
                  'lib/components/sidebar/*',
                  'lib/components/tests/*',
                  'lib/components/user-manager/*',
                  'lib/interfaces/*',
                  'pages/*',
                  'public/*',
                  'public/icons/*',
                  'redux/*',
                  'redux/actions/*',
                  'redux/reducers/*',
                  'redux/store/*',
                  'styles/*',
                  'tests/intergration/Dockerfile',
                  'tests/intergration/Dockerfile',
                  'tests/intergration/Dockerfile',
                  'tests/intergration/Dockerfile',
                  'tests/intergration/cypress.json',
                  'tests/intergration/cypress.json',
                  'tests/intergration/cypress.json',
                  'tests/intergration/cypress.json',
                  'tests/intergration/cypress/fixtures/*',
                  'tests/intergration/cypress/integration/cyc/*',
                  'tests/intergration/cypress/integration/data/*',
                  'tests/intergration/cypress/plugins/*',
                  'tests/intergration/cypress/results/*',
                  'tests/intergration/cypress/support/*',
                  'tests/intergration/old-test.js',
                  'tests/intergration/old-test.js',
                  'tests/intergration/old-test.js',
                  'tests/intergration/old-test.js',
                  'tests/intergration/package.json',
                  'tests/intergration/package.json',
                  'tests/intergration/package.json',
                  'tests/intergration/package.json',
                  'tests/unit/*',
                  'tests/unit/pages/*',
                  'theme/*'],
 'cyc_next_app.server': ['build/*', 'ls/*', 'routes/*']}

install_requires = \
['cnextlib>=0.2.0,<0.3.0',
 'jupyter-client>=7.2.2,<8.0.0',
 'jupyterlab>=3.3.4,<4.0.0',
 'matplotlib>=3.5.1,<4.0.0',
 'mlflow>=1.25.1,<2.0.0',
 'multipledispatch>=0.6.0,<0.7.0',
 'netron>=5.7.8,<6.0.0',
 'pandas==1.3.5',
 'plotly>=5.7.0,<6.0.0',
 'protobuf==3.20.1',
 'python-language-server>=0.36.2,<0.37.0',
 'requests>=2.27.1,<3.0.0',
 'send2trash>=1.8.0,<2.0.0',
 'sentry-sdk>=1.5.12,<2.0.0',
 'simplejson>=3.17.6,<4.0.0',
 'zmq>=0.0.0,<0.0.1']

entry_points = \
{'console_scripts': ['cnext-init = cyc_next_app.__main__:main',
                     'cnext-path = cyc_next_app.__main__:path',
                     'cnext-run = cyc_next_app.__main__:start']}

setup_kwargs = {
    'name': 'cnext-test',
    'version': '0.4.19',
    'description': 'The data-centric workspace for AI & DS',
    'long_description': '# CNext Instructions\n\n[Website] - [Documentation] - [Docker Image] - [Overview Video]\n\n## Setup and run the CNext workspace\n\nCNext is a data-centric workspace for DS and AI. Our workspace is meant to consolidate the most common tasks performed by data scientists and ML engineers. At a high level our workspace allows for:\n\n-   Data exploration & transformation\n-   Model development / exploration\n-   Production code generation\n-   Dashboard & App Generation\n-   Experiment Management\n\n## Features\n\n-   Interactive Python coding envrionment with native Python output (think Jupyter replacement)\n-   Smart code suggestion (categorical values and column names)\n-   Interactive data exploration\n-   Automative visualizations\n-   Experiment and model management\n-   Instant dashboarding\n\n## Installation via Docker\n\nStep 1: `Download` [cnext] sample project here and `extract`\n\nStep 2: `setup` [Docker] on your computer\n\nStep 3: Run `docker login` login with your docker account\n\nStep 4: Point destination command to `cnext` directory\n\nStep 5: Run command `docker-compose up -d`\n\n-   The web application will launch at : `http://localhost:CLIENT_PORT` or `http://127.0.0.1:CLIENT_PORT/` (CLIENT_PORT default is 3000)\n-   To stop the application: `docker-compose down`\n-   Note: Pay attention at `CLIENT_PORT`, and `SERVER_PORT` in `.env` file (you will have to change these ports if you already use them on your machine, by default they are 3000 and 4000 respectively)\n\n## Installation via Pip\n\nStep 1: `Download` [cnext] sample project here. This folder will be use as an input in Step 4\n\nStep 2: Make sure `Nodejs` is available in your computer (try `npm --version` and make sure it work)\n\nStep 3: `run` command `pip install cnext`\n\nStep 4: `run` command `cnext-init`\n\n-   Input `Enter path to the cnext sample project created in Step 1` and hit `Enter` (Example `C:/Skywalker`)\n    \u200b\n\nStep 6 `run` command `cnext-run`\n\n-   Web application will launch at : `http://localhost:CLIENT_PORT` or `http://127.0.0.1:CLIENT_PORT/` (CLIENT_PORT default is 3000)\n-   Stop application: `Ctrl + c | Command + c`\n-   Note: Pay attention at `CLIENT_PORT`, and `SERVER_PORT` in `.env` file (you will have to change these ports if you already use them on your machine, by default they are 3000 and 4000 respectively)\n\n## Release Note\n\n# v0.4.18 - 0.4.19\n\n##### NEW\n\n-   Show failed execution status marker\n\n# v0.4.14 - v0.4.16\n\n##### NEW\n\n-   Add configs for dataframe manager.\n-   Implement grouped lines folding\n\n##### IMPROVED\n\n-\n\n##### FIXED\n\n-   Fix bugs and improve stability for model visualization.\n\n# v0.4.12 - v0.4.13\n\n##### NEW\n\n-   Allow code to be executed on any file.\n-   Improve code suggestion behavior.\n-   Load config dynamically from "config.json".\n\n##### IMPROVED\n\n-\n\n##### FIXED\n\n-\n\n# v0.4.10\n\n##### NEW\n\n-   Assign result of a group to only the first line.\n-   Add support for full-html-page results using iframe.\n-   Add support for .sql and .json files.\n\n##### IMPROVED\n\n-   Improve the ability to support results with javascript. you can now run bokeh plot.\n\n##### FIXED\n\n-\n\n## License\n\nCopyright 2022 CycAI Inc.\n\u200b\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\u200b\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\u200b\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.\n\n[website]: https://www.cnext.io/\n[docker image]: https://hub.docker.com/r/cycai/cnext\n[documentation]: https://internal-lace-ae4.notion.site/Product-Documentation-0dd58ea1cfe14dfab3666c5ec633ae96\n[overview video]: https://youtu.be/5eWPkQIUfZw\n[cnext]: https://drive.google.com/file/d/1w4MU3nr0E14PAS_5NmruuoMSfQeE5MLK\n[docker]: https://www.docker.com/products/docker-desktop/\n',
    'author': 'CycAI Inc',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://cyc-ai.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9',
}


setup(**setup_kwargs)
