import React from "react";
import { Logo as StyledLogo, LogoIcon } from "../StyledComponents";

const Logo = () => {
    return (
        <StyledLogo>
            <LogoIcon alt="" src="/favicon.ico" />
        </StyledLogo>
    );
};

export default Logo;
